#include "CGridMapViewer.h"

// \J[`}N
#define COLOR_WHITE CV_RGB(255, 255, 255)
#define COLOR_BLACK CV_RGB(0, 0, 0)
#define COLOR_RED   CV_RGB(255, 0, 0)
#define COLOR_GREEN CV_RGB(0, 255, 0)
#define COLOR_BLUE  CV_RGB(0, 0, 255)
#define COLOR_GRAY  CV_RGB(120, 120, 120)
#define COLOR_ROBOT CV_RGB(168, 230, 29) // {bg\F
#define COLOR_OBSTACL CV_RGB(128, 0, 0) // Q\F

namespace RobotSystem{
	//-------------------- ImageViewer̎ ----------------------------
	ImageViewer::ImageViewer()
		:_wndname("ImageViewer"), _wndimg(0),_wndw(640), _wndh(480)
	{
		_wndimg = cvCreateImage(cvSize(_wndw, _wndh), IPL_DEPTH_8U, 3);
	}
	
	ImageViewer::ImageViewer(string wndname)
		:_wndname(wndname), _wndimg(0),_wndw(640), _wndh(480)
	{
		_wndimg = cvCreateImage(cvSize(_wndw, _wndh), IPL_DEPTH_8U, 3);
	}

	ImageViewer::ImageViewer(int w, int h)
		:_wndname("ImageViewer"), _wndimg(0),_wndw(w), _wndh(h)
	{
		_wndimg = cvCreateImage(cvSize(_wndw, _wndh), IPL_DEPTH_8U, 3);
	}

	ImageViewer::ImageViewer(string wndname, int w, int h)
		:_wndname(wndname), _wndimg(0),_wndw(w), _wndh(h)
	{
		_wndimg = cvCreateImage(cvSize(_wndw, _wndh), IPL_DEPTH_8U, 3);
	}

	ImageViewer::~ImageViewer()
	{
		cvReleaseImage(&_wndimg);
		closeWnd();
	}

	void ImageViewer::setWndSize(int w, int h){
		if(_wndw != w || _wndh != h){
			cvReleaseImage(&_wndimg);
			cvCreateImage(cvSize(w, h), IPL_DEPTH_8U, 3);
			cvResizeWindow(_wndname.c_str(), w, h);
		}
		_wndw=w; _wndh=h;
	}

	void ImageViewer::setWndName(string name){
		_wndname=name;
	}

	void ImageViewer::openWnd(){
		cvNamedWindow(_wndname.c_str());
		cvResizeWindow(_wndname.c_str(), _wndw, _wndh);
	}
	void ImageViewer::closeWnd(){
		cvDestroyWindow(_wndname.c_str());
	}

	// 摜̕\
	void ImageViewer::display(){
		cvShowImage(_wndname.c_str(), _wndimg);
	}
	void ImageViewer::display(void *img){
		memcpy((unsigned char*)_wndimg->imageData, (unsigned char*)img, _wndw*_wndh*3);
		cvShowImage(_wndname.c_str(), _wndimg);
	}

	// \Ɖ摜XV
	int ImageViewer::waitKeyDisplay(int msec){
		display();
		return waitKey(msec);
	}
	int ImageViewer::waitKeyDisplay(void *img, int msec){
		display(img);
		return waitKey(msec);
	}

	// 摜XVƃL[
	int ImageViewer::waitKey(int msec){
		return cvWaitKey(msec);
	}

	//---------------------- GridMapViewer̎ ------------------------------
	// RXgN^
	GridMapViewer::GridMapViewer()
		:_map(0), _gpixel(10)
    {
		_wndname= "GridMapViewer";
	}
	GridMapViewer::GridMapViewer(int rsize)
		:_map(0), _gpixel(10), _rsize(rsize)
	{
		_wndname= "GridMapViewer";
	}


	// fXgN^
	GridMapViewer::~GridMapViewer(){
	}

	// {bga̐ݒ
	// @param rsize : {bga
	void GridMapViewer::setRobotSize(int rsize){
		_rsize = rsize;
	}

	void GridMapViewer::onGridLine(bool on){
		option_flag.set( GRIDLINE, on );
	}

	// Obh}bv̕\
	void GridMapViewer::display(GridMap<int, GridMapGenerator>& map){
		const int ogval(-1); //QObhl

		_map=&map;
		// Obh}bv̎擾
		int grow(0), gcol(0), gsize(0);
		map.getGridMapInformation(grow, gcol, gsize);
		GridMapData<int> *mapdata(0);
		map.getCurrentMapData(mapdata);

		// Obh\[pixel]̌
		_gpixel = _wndh/gcol;
		if(_gpixel*grow>_wndw-1){
			_gpixel = _wndw/grow;
		}

		// \摜NA
		cvSet(_wndimg, COLOR_WHITE);

		// Q, {bg݂̑ObhhԂ
		for(int x(0); x<grow; ++x){
			for(int y(0); y<gcol; ++y){
				if((*mapdata)[x][y] == ogval){
					drawObstacls(x, y);
				}
			}
		}
		Position2D<int> rpos;
		map.getCurrentRobotPosition(rpos);
		map.getCurrentRobotPositionOnGrid(rpos.x, rpos.y);
		drawRobot(rpos, gsize);

		// ̕`
		drawAxis(grow, gcol);

		// }bvÖ̓hԂ
		cvRectangle( _wndimg, cvPoint(grow*_gpixel, 0), cvPoint(_wndw, _wndh), COLOR_GRAY, -1);
		cvRectangle( _wndimg, cvPoint(0, gcol*_gpixel), cvPoint(_wndw, _wndh), COLOR_GRAY, -1);
		
		// 摜𔽓]
		cvFlip(_wndimg, NULL, 0);
		ImageViewer::display();
	}

	// ̕`
	void GridMapViewer::drawAxis(int grow, int gcol){
		if( option_flag[GRIDLINE] ){
		// ObhC`
		for(int x(0); x<grow*_gpixel; x+=_gpixel){
			cvDrawLine( _wndimg, cvPoint(x, 0), cvPoint(x, _wndh), COLOR_BLACK, 1 );
		}
		for(int y(0); y<gcol*_gpixel; y+=_gpixel){
			cvDrawLine( _wndimg, cvPoint(0, y), cvPoint(_wndw, y), COLOR_BLACK, 1 );
		}
		}
		

		// E`
		cvDrawLine( _wndimg, cvPoint(0, 0), cvPoint( grow*_gpixel, 0), COLOR_BLACK, 3);
		cvDrawLine( _wndimg, cvPoint(0, 0), cvPoint( 0, gcol*_gpixel), COLOR_BLACK, 3);
		cvDrawLine( _wndimg, cvPoint(grow*_gpixel, 0), cvPoint( grow*_gpixel, gcol*_gpixel), COLOR_BLACK, 3);
		cvDrawLine( _wndimg, cvPoint(0, gcol*_gpixel), cvPoint( grow*_gpixel, gcol*_gpixel), COLOR_BLACK, 3);

		// 摜֕`
		cvDrawLine( _wndimg, cvPoint(0,_gpixel*0.5), cvPoint(_wndw, _gpixel*0.5), COLOR_RED, 3 ); //x-axis
		cvDrawLine( _wndimg, cvPoint(_gpixel*0.5,0), cvPoint(_gpixel*0.5, _wndh), COLOR_BLUE, 3 ); //y-axis
	}

	// {bg̕`
	void GridMapViewer::drawRobot(Position2D<int> rpos, int gsize){
		// {bga̕`
		double r((double)_rsize/gsize*_gpixel);
		//rpos.x=d2i((double)rpos.x/gsize); rpos.y=d2i((double)rpos.y/gsize); // Obh֕ϊ
		cvCircle( _wndimg, cvPoint( (rpos.x+0.5)*_gpixel, (rpos.y+0.5)*_gpixel ), r, COLOR_ROBOT, -1);
		// {bgSObh̓hԂ
		cvRectangle( _wndimg, cvPoint(rpos.x*_gpixel, rpos.y*_gpixel), cvPoint((rpos.x+1)*_gpixel, (rpos.y+1)*_gpixel), COLOR_ROBOT, -1);
		// {bg̕`
		r=r*sqrt(2.0);
		cvDrawLine( _wndimg, cvPoint((rpos.x+0.5)*_gpixel, (rpos.y+0.5)*_gpixel),
							 cvPoint((rpos.x+0.5)*_gpixel+cvRound(r*cos(rpos.w.toRad())), (rpos.y+0.5)*_gpixel+cvRound(r*sin(rpos.w.toRad()))),
					COLOR_BLACK, 1);
	}

	// Q̕`
	void GridMapViewer::drawObstacls(int gx, int gy){
		// QObh̓hԂ
		cvRectangle( _wndimg, cvPoint(gx*_gpixel, gy*_gpixel), cvPoint((gx+1)*_gpixel, (gy+1)*_gpixel), COLOR_OBSTACL, -1);
	}


	int GridMapViewer::waitKeyDisplay(GridMap<int, GridMapGenerator>& map, int msec){
		display(map);
		return waitKey(msec);
	}

	//-----------------------------------------------------------------
	/*
	PGridMapViewer::PGridMapViewer()
    {
		_wndname = "GridMapViewer";
	}

	// fXgN^
	PGridMapViewer::~PGridMapViewer(){
	}
	// 摜
	void PGridMapViewer::display(GridMap& map){
		const int ogval(-1); //QObhl
		const int igval(0xffff);//issObhl

		_map=&map;
		// Obh}bv̎擾
		int grow(0), gcol(0), gsize(0);
		map.getGridMapInformation(grow, gcol, gsize);
		int rsize = map.getRobotSize();

		// Obh\[pixel]̌
		if(_wndw*gcol > _wndh*grow){
			_gpixel = _wndh/gcol;
		}else{
			_gpixel = _wndw/grow;
		}

		// \摜NA
		cvSet(_wndimg, COLOR_WHITE);

		CvFont dfont;
		cvInitFont(&dfont, CV_FONT_HERSHEY_PLAIN, 1.0f, 1.0f);
		char str[125];

		// Q, {bg݂̑ObhhԂ
		for(int x(0); x<grow; ++x){
			for(int y(0); y<gcol; ++y){
				if(map[x][y] == igval){
					// Bs\Obh̓hԂ
					cvRectangle( _wndimg, cvPoint(x*_gpixel, y*_gpixel), cvPoint((x+1)*_gpixel, (y+1)*_gpixel), COLOR_OBSTACL, -1);
				}else{
					sprintf(str,"%d", map[x][y]);
					cvPutText( _wndimg, str, cvPoint((x+0.5)*_gpixel, (y+0.5)*_gpixel), &dfont, COLOR_BLACK);
				}
			}
		}
		drawAxis(grow, gcol);
		
		// 摜𔽓]
		cvFlip(_wndimg, NULL, 0);
		ImageViewer::display();
	}
	*/
};
