// -*- C++ -*-
/*!
 * @file  GridMap.cpp
 * @brief Grid map genarator
 * @date $Date$
 *
 * $Id$
 */

#include "GridMap.h"

// Module specification
// <rtc-template block="module_spec">
static const char* gridmap_spec[] =
  {
    "implementation_id", "GridMap",
    "type_name",         "GridMap",
    "description",       "Grid map genarator",
    "version",           "0.8.0",
    "vendor",            "Kenich Ohara, Osaka Univ.",
    "category",          "Book",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    "exec_cxt.periodic.rate", "60.0",
    // Configuration variables
    "conf.default.GridSize", "300",
    "conf.default.GridRow", "30",
    "conf.default.GridCol", "30",
	"conf.default.GridVoteNum", "10",
	"conf.default.RobotRadius", "150",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
GridMap::GridMap(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_opIn("ObstaclePosition", m_op),
    m_rpIn("RobotPosition", m_rp),
    m_MapControlPort("MapControl")

    // </rtc-template>
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("ObstaclePosition", m_opIn);
  registerInPort("RobotPosition", m_rpIn);
  
  // Set OutPort buffer
  
  // Set service provider to Ports
  m_MapControlPort.registerProvider("MapCtr", "MapControl", m_MapCtr);
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  registerPort(m_MapControlPort);
  
  // </rtc-template>


}

/*!
 * @brief destructor
 */
GridMap::~GridMap()
{
}



RTC::ReturnCode_t GridMap::onInitialize()
{
	std::cout << "##### GridMap #####" << std::endl;
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("GridSize", m_gsize, "300");
  bindParameter("GridRow", m_grow, "30");
  bindParameter("GridCol", m_gcol, "30");
  bindParameter("GridVoteNum", m_gvoteNum, "10");  
  bindParameter("RobotRadius", m_rrad, "150");

  // {bgʒupobt@̊m
  m_rp.data.length(3);
  
  m_MapCtr.lockService();
  // </rtc-template>
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t GridMap::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GridMap::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GridMap::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t GridMap::onActivated(RTC::UniqueId ec_id)
{
	// Obh}bv̏
	_gmg.setVoteNum(m_gvoteNum);
	_gmg.setGridMapSize(m_grow, m_gcol, m_gsize);
	_map = new RobotSystem::GridMap<int, RobotSystem::GridMapGenerator>(&_gmg);

	// {bgʒȕ
	m_rp.data[0]=0; m_rp.data[1]=0; m_rp.data[2]=0;

	// Obh}bvEBhẼI[v
	_gmv.setRobotSize(m_rrad);
	_gmv.openWnd();

	m_MapCtr.unlockService();
  return RTC::RTC_OK;
}



RTC::ReturnCode_t GridMap::onDeactivated(RTC::UniqueId ec_id)
{
	// Obh}bv̏I
	delete _map;

	// Obh}bv\EBhẼN[Y
	_gmv.closeWnd();

	m_MapCtr.lockService();
  return RTC::RTC_OK;
}



RTC::ReturnCode_t GridMap::onExecute(RTC::UniqueId ec_id)
{
	RobotSystem::Position2D<int> rp;      // {bgʒu
	vector<RobotSystem::Point2D<int>> op; // Qʒu
	char key;
	static bool lineon;

	// {bgʒu擾
	if(m_rpIn.isNew()){
		m_rpIn.read();
	}
	rp.x = m_rp.data[0]; rp.y = m_rp.data[1]; rp.w.setDeg(m_rp.data[2]);
	
	// ZTQ擾
	if(m_opIn.isNew()){
		m_opIn.read();
		op.resize(m_op.data.length()/2);
		for(int i(0); i<op.size(); ++i){
			op[i].x=m_op.data[2*i];
			op[i].y=m_op.data[2*i+1];
		}
		// }bvXV
		_map->MapRenewal(rp, op);
		m_MapCtr.setMap(*_map);

		// }bv\
		key=_gmv.waitKeyDisplay(*_map, 10);
	}else{
		// }bv\
		RobotSystem::ImageViewer *_imv = &_gmv;
		key=_imv->waitKeyDisplay(10);
	}
	if(key == 'l'){
		if(lineon) lineon=false;
		else lineon=true;
		_gmv.onGridLine(lineon);
	}

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t GridMap::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GridMap::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GridMap::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GridMap::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GridMap::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void GridMapInit(RTC::Manager* manager)
  {
    RTC::Properties profile(gridmap_spec);
    manager->registerFactory(profile,
                             RTC::Create<GridMap>,
                             RTC::Delete<GridMap>);
  }
  
};
