// -*- C++ -*-
/*!
 * @file  PositionSensor.cpp
 * @brief Position Detection Sensor in Indoor Environment
 * @date $Date$
 *
 * $Id$
 */

#include "PositionSensor.h"

// Module specification
// <rtc-template block="module_spec">
static const char* positionsensor_spec[] =
  {
    "implementation_id", "PositionSensor",
    "type_name",         "PositionSensor",
    "description",       "Position Detection Sensor in Indoor Environment",
    "version",           "1.0.0",
    "vendor",            "Kenichi OHARA, Osaka Univ.",
    "category",          "Book",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    "exec_cxt.periodic.rate", "1000.0",
    // Configuration variables
    "conf.default.ComPort", "COM1",
    "conf.default.BaudRate", "115200",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
PositionSensor::PositionSensor(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_LocalPositionOut("LocalPosition", m_LocalPosition),

    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  
  // Set OutPort buffer
  registerOutPort("LocalPosition", m_LocalPositionOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

/*!
 * @brief destructor
 */
PositionSensor::~PositionSensor()
{
}



RTC::ReturnCode_t PositionSensor::onInitialize()
{
	
  std::cout << "##### PositionSensor #####" << std::endl;

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("ComPort", m_ComPort, "COM1");
  bindParameter("BaudRate", m_BaudRate, "115200");
  
  // </rtc-template>
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t PositionSensor::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t PositionSensor::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t PositionSensor::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t PositionSensor::onActivated(RTC::UniqueId ec_id)
{
	//PositionSensorւ̐ڑ
	if(!ps.Connect(m_ComPort.c_str(), m_BaudRate)){
		return RTC::RTC_ERROR;
	}
	return RTC::RTC_OK;
}



RTC::ReturnCode_t PositionSensor::onDeactivated(RTC::UniqueId ec_id)
{
	//PositionSensor̐ؒf
	ps.Disconnect();

	return RTC::RTC_OK;
}



RTC::ReturnCode_t PositionSensor::onExecute(RTC::UniqueId ec_id)
{
	double x,y,theta;

	//PositionSensor̃f[^̎擾яo
	ps.getLocalPosition(&x,&y,&theta);
	m_LocalPosition.data.length(3);
	m_LocalPosition.data[0] = x;
	m_LocalPosition.data[1] = y;
	m_LocalPosition.data[2] = theta;
	m_LocalPositionOut.write();
	std::cout<<"(X[mm],Y[mm],theta[degree])=("<<x<<","<<y<<","<<theta<<")"<<std::endl;
	
	return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t PositionSensor::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t PositionSensor::onError(RTC::UniqueId ec_id)
{
	std::cout<<"w|[gio[{[[gCmFĂD"<<std::endl;
	return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t PositionSensor::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t PositionSensor::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t PositionSensor::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void PositionSensorInit(RTC::Manager* manager)
  {
    RTC::Properties profile(positionsensor_spec);
    manager->registerFactory(profile,
                             RTC::Create<PositionSensor>,
                             RTC::Delete<PositionSensor>);
  }
  
};


