// -*- C++ -*-
/*!
 * @file  PositionCheck.cpp
 * @brief Position Check Component for Position Sensor
 * @date $Date$
 *
 * $Id$
 */

#include "PositionCheck.h"

// Module specification
// <rtc-template block="module_spec">
static const char* positioncheck_spec[] =
  {
    "implementation_id", "PositionCheck",
    "type_name",         "PositionCheck",
    "description",       "Position Check Component for Position Sensor",
    "version",           "1.0.0",
    "vendor",            "Kenichi OHARA, Osaka Univ.",
    "category",          "book",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    "exec_cxt.periodic.rate", "1000.0",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
PositionCheck::PositionCheck(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_LocalPositionIn("LocalPosition", m_LocalPosition),

    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("LocalPosition", m_LocalPositionIn);
  
  // Set OutPort buffer
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

/*!
 * @brief destructor
 */
PositionCheck::~PositionCheck()
{
}


/*
RTC::ReturnCode_t PositionCheck::onInitialize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t PositionCheck::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t PositionCheck::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t PositionCheck::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t PositionCheck::onActivated(RTC::UniqueId ec_id)
{
	image = cvCreateImage(cvSize(IMAGE_SIZE,IMAGE_SIZE), IPL_DEPTH_8U, 1);

	cvNamedWindow("Position Sensor Viewer");
	return RTC::RTC_OK;
}



RTC::ReturnCode_t PositionCheck::onDeactivated(RTC::UniqueId ec_id)
{
	if(image != NULL){
		cvReleaseImage(&image);
	}
	cvDestroyWindow("Position Sensor Viewer");
	return RTC::RTC_OK;
}


RTC::ReturnCode_t PositionCheck::onExecute(RTC::UniqueId ec_id)
{
	int x(0), y(0);
	double theta(0);
	if(m_LocalPositionIn.isNew()){
		m_LocalPositionIn.read();

		//̃NA
		memset(image->imageData,0,image->imageSize);

		//f[^̃Rs[
		x = IMAGE_SIZE/2 - (int)(m_LocalPosition.data[0]/MAGNIFICATION);
		y = IMAGE_SIZE/2 - (int)(m_LocalPosition.data[1]/MAGNIFICATION);
		theta = m_LocalPosition.data[2]*3.1415/180;

		cvCircle(image, cvPoint(x,y), RADIUS, CV_RGB(255,255,255), 2);
			
		//W̏
		cvLine(image,cvPoint(IMAGE_SIZE/2,0),cvPoint(IMAGE_SIZE/2,IMAGE_SIZE),CV_RGB(255,255,255),2);
		cvLine(image,cvPoint(0,IMAGE_SIZE/2),cvPoint(IMAGE_SIZE, IMAGE_SIZE/2),CV_RGB(255,255,255),2);

		//xNg̏
		cvLine(image,cvPoint(x,y),cvPoint(x+100*cos(theta),y+100*sin(theta)),CV_RGB(255,255,255),2);
		cvShowImage("Position Sensor Viewer", image);
		cvWaitKey(10);
	}
	return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t PositionCheck::onAbdorting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t PositionCheck::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t PositionCheck::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t PositionCheck::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t PositionCheck::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void PositionCheckInit(RTC::Manager* manager)
  {
    RTC::Properties profile(positioncheck_spec);
    manager->registerFactory(profile,
                             RTC::Create<PositionCheck>,
                             RTC::Delete<PositionCheck>);
  }
  
};


