// -*- C++ -*-
/*!
 * @file  ScanningRangeSensor.cpp
 * @brief ScanningRangeSensor for URG
 * @date $Date$
 *
 * $Id$
 */

#include "ScanningRangeSensor.h"

// Module specification
// <rtc-template block="module_spec">
static const char* scanningrangesensor_spec[] =
  {
    "implementation_id", "ScanningRangeSensor",
    "type_name",         "ScanningRangeSensor",
    "description",       "ScanningRangeSensor for URG",
    "version",           "1.0.0",
    "vendor",            "VenderName",
    "category",          "Book",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    "exec_cxt.periodic.rate", "1000.0",
	// Configuration variables
    "conf.default.ComPort", "COM1",
    "conf.default.BaudRate", "115200",
    "conf.default.XaxisDirection", "right",
    "conf.default.Offset_x", "60",
    "conf.default.Offset_y", "0",
    "conf.default.Offset_theta", "0",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
ScanningRangeSensor::ScanningRangeSensor(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_DistanceDataOut("Distance", m_DistanceData),

    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  
  // Set OutPort buffer
  registerOutPort("Distance", m_DistanceDataOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}


/*!
 * @brief destructor
 */
ScanningRangeSensor::~ScanningRangeSensor()
{
}



RTC::ReturnCode_t ScanningRangeSensor::onInitialize()
{
  printf("##### ScanningRangeSensor #####\n");
  m_DistanceData.data.length(769*3);
  bindParameter("ComPort", m_ComPort, "COM1");
  bindParameter("BaudRate", m_BaudRate, "115200");
  bindParameter("XaxisDirection", m_XaxisDirection, "right");
  bindParameter("Offset_x", m_Offset_x, "60");
  bindParameter("Offset_y", m_Offset_y, "0");
  bindParameter("Offset_theta", m_Offset_theta, "0");

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t ScanningRangeSensor::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ScanningRangeSensor::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ScanningRangeSensor::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t ScanningRangeSensor::onActivated(RTC::UniqueId ec_id)
{
  printf("onActived\n");

  //URGʕɑ΂ĂxW̌ݒ(URGWn̐ݒ)
  _rf.setXDirection(m_XaxisDirection);

  //PositionSensorւ̐ڑ
  _rf.Connect(m_ComPort.c_str(), m_BaudRate);
  return RTC::RTC_OK;
}


RTC::ReturnCode_t ScanningRangeSensor::onDeactivated(RTC::UniqueId ec_id)
{
  printf("onDeactived\n");

  //PositionSensorؒf
  _rf.Disconnect();
  return RTC::RTC_OK;
}


RTC::ReturnCode_t ScanningRangeSensor::onExecute(RTC::UniqueId ec_id)
{
  long x[769],y[769];
  int i=0,k=0;

  std::cout << "----------------------GetDataStart--------------------------------------" <<std::endl;

  //T[`e_xW,yW,zWl
  _rf.getMeasuredData(x,y);

  std::cout << "----------------------GetDataFinish-------------------------------------" <<std::endl;

/*
@//e_̊lf[^\
  std::cout << "------------------------------------------------------------" <<std::endl;
  for(int i(0); i<769; ++i){
	std::cout<<"(x,y,z)=("<<x[i]<<","<<y[i]<<")"<<std::endl;
  }
  std::cout << "------------------------------------------------------------" << std::endl;
*/

  //lf[^o̓|[gɐݒu
  for(i=0;i<769;i++){
	  m_DistanceData.data[k++]=x[i]+m_Offset_x*cos(M_PI*m_Offset_theta/180.0);
	  m_DistanceData.data[k++]=y[i]+m_Offset_y*sin(M_PI*m_Offset_theta/180.0);
  }

  //o̓|[glf[^o
  if(!m_DistanceDataOut.write()){
	  std::cout << "send error!: distance data has not sent." <<std::endl;
  }

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t ScanningRangeSensor::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ScanningRangeSensor::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ScanningRangeSensor::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ScanningRangeSensor::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ScanningRangeSensor::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void ScanningRangeSensorInit(RTC::Manager* manager)
  {
    RTC::Properties profile(scanningrangesensor_spec);
    manager->registerFactory(profile,
                             RTC::Create<ScanningRangeSensor>,
                             RTC::Delete<ScanningRangeSensor>);
  }
  
};


