// -*- C++ -*-
/*!
 * @file  SensorManager.cpp
 * @brief 自己位置推定結果統合のためのRTコンポーネント
 * @date $Date$
 *
 * $Id$
 */

#include "SensorManager.h"

inline int d2i(double val){
	if(val>=0.0){
		val+=0.5;
	}else{
		val-=0.5;
	}
	return (int)val;
}

// Module specification
// <rtc-template block="module_spec">
static const char* sensormanager_spec[] =
  {
    "implementation_id", "SensorManager",
    "type_name",         "SensorManager",
//    "description",       "自己位置推定結果統合",
    "description",       "hogehoge",
    "version",           "0.4.0",
    "vendor",            "Kenich Ohara, Osaka Univ.",
    "category",          "Book",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    "exec_cxt.periodic.rate", "1.0",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
SensorManager::SensorManager(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_odometryIn("Odometry", m_odometry),
    m_localposIn("LocalPosition", m_localpos),
    m_robotposOut("RobotPosition", m_robotpos)

    // </rtc-template>
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("Odometry", m_odometryIn);
  registerInPort("LocalPosition", m_localposIn);
  
  // Set OutPort buffer
  registerOutPort("RobotPosition", m_robotposOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

/*!
 * @brief destructor
 */
SensorManager::~SensorManager()
{
}



RTC::ReturnCode_t SensorManager::onInitialize()
{
	std::cout << "##### SensorManager #####" << std::endl;

	// アウトポートのバッファを確保
	m_robotpos.data.length(3);
  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t SensorManager::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t SensorManager::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t SensorManager::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t SensorManager::onActivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}



RTC::ReturnCode_t SensorManager::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}



RTC::ReturnCode_t SensorManager::onExecute(RTC::UniqueId ec_id)
{
	vector<double> vecodo, veclpos;
	vector<double> *pvecodo(0), *pveclpos(0);

	if (m_odometryIn.isNew()){
		m_odometryIn.read();
		vecodo.push_back((double)m_odometry.data[0]);
		vecodo.push_back((double)m_odometry.data[1]);
		vecodo.push_back((double)m_odometry.data[2]);
		pvecodo = &vecodo;
		std::cout << "reading the pose from odometry port." << std::endl;
	}
	if (m_localposIn.isNew()){
		m_localposIn.read();
		veclpos.push_back((double)m_localpos.data[0]);
		veclpos.push_back((double)m_localpos.data[1]);
		veclpos.push_back((double)m_localpos.data[2]);
		pveclpos = &veclpos;
		std::cout << "reading the pose from local position port." << std::endl;
	}
	if (pvecodo != 0 || pveclpos != 0){
		std::cout << "marging the pose." << std::endl;
		vector<double> margepos = _sm.margePosition(pvecodo, pveclpos);
		m_robotpos.data[0]=d2i(margepos[0]);
		std::cout << "x coordinate : " << m_robotpos.data[0] << "[mm]" << std::endl;
		m_robotpos.data[1]=d2i(margepos[1]);
		std::cout << "y coordinate : " << m_robotpos.data[1] << "[mm]" << std::endl;
		m_robotpos.data[2]=d2i(margepos[2]);
		std::cout << "pose angle : " << m_robotpos.data[2] << "[deg]" << std::endl;
		m_robotposOut.write();
	}
	
  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t SensorManager::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t SensorManager::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t SensorManager::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t SensorManager::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t SensorManager::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void SensorManagerInit(RTC::Manager* manager)
  {
    RTC::Properties profile(sensormanager_spec);
    manager->registerFactory(profile,
                             RTC::Create<SensorManager>,
                             RTC::Delete<SensorManager>);
  }
  
};


