// -*- C++ -*-
/*!
 * @file  URGView.cpp
 * @brief URG Data Viewer
 * @date $Date$
 *
 * $Id$
 */

#include "URGView.h"

// Module specification
// <rtc-template block="module_spec">
static const char* urgview_spec[] =
  {
    "implementation_id", "URGView",
    "type_name",         "URGView",
    "description",       "URG Data Viewer",
    "version",           "1.0.0",
    "vendor",            "Kenichi OHARA, Osaka Univ.",
    "category",          "book",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    "exec_cxt.periodic.rate", "1000.0",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
URGView::URGView(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_DistanceIn("Distance", m_Distance),

    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("Distance", m_DistanceIn);
  
  // Set OutPort buffer
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

	image = NULL;

}

/*!
 * @brief destructor
 */
URGView::~URGView()
{
}


/*
RTC::ReturnCode_t URGView::onInitialize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t URGView::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t URGView::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t URGView::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t URGView::onActivated(RTC::UniqueId ec_id)
{
	//\pC[W̍쐬
	image = cvCreateImage(cvSize(IMAGE_SIZE,IMAGE_SIZE),IPL_DEPTH_8U, 1);
	showimage = cvCreateImage(cvSize(IMAGE_SIZE,IMAGE_SIZE),IPL_DEPTH_8U, 1);
	cvNamedWindow("URG Data Viewer");
	return RTC::RTC_OK;
}



RTC::ReturnCode_t URGView::onDeactivated(RTC::UniqueId ec_id)
{
	if(image != NULL){
		cvReleaseImage(&image);
	}
	if(showimage !=NULL){
		cvReleaseImage(&showimage);
	}
	cvDestroyWindow("URG Data Viewer");
	return RTC::RTC_OK;
}



RTC::ReturnCode_t URGView::onExecute(RTC::UniqueId ec_id)
{
	int x(0),y(0),z(0);
	int pt(0);

	if(m_DistanceIn.isNew()){
		memset(image->imageData,0,image->imageSize);
		//f[^|[g̃f[^̓ǂݍ
		m_DistanceIn.read();


		//f[^̃Rs[
		for(int i(0);i<m_Distance.data.length();i+=3){
			x = m_Distance.data[i]/REDUCTION_RATIO;
			y = m_Distance.data[i+1]/REDUCTION_RATIO;
			z = m_Distance.data[i+2];
			if(abs(x) > IMAGE_SIZE/2){
				if( x > 0)
					x = IMAGE_SIZE/2;
				else
					x = -IMAGE_SIZE/2;
			}
			if(abs(y) > IMAGE_SIZE/2){
				if( y > 0)
					y = IMAGE_SIZE/2;
				else
					y = -IMAGE_SIZE/2;
			}

			//YW𔒂D
			image->imageData[(image->width/2 - x) + ( image->height/2 - y) * IMAGE_SIZE] = 255; 
		}
		cvDilate(image, showimage, NULL, DILATE); 
		cvShowImage("URG Data Viewer", showimage);
		cvWaitKey(10);
	}
	return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t URGView::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t URGView::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t URGView::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t URGView::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t URGView::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void URGViewInit(RTC::Manager* manager)
  {
    RTC::Properties profile(urgview_spec);
    manager->registerFactory(profile,
                             RTC::Create<URGView>,
                             RTC::Delete<URGView>);
  }
  
};


